import { FormControl, FormGroup, FormArray, FormBuilder, ReactiveFormsModule } from '@angular/forms';
import { Injectable, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AutomaticFormBuilder {
    /**
     * @param {?} formbuilder
     */
    constructor(formbuilder) {
        this.group = formbuilder.group;
        this.control = formbuilder.control;
        this.array = formbuilder.array;
    }
    /**
     * @param {?} options
     * @return {?}
     */
    setupForm(options) {
        /** @type {?} */
        let formGroup;
        if (this.isObject(options)) {
            formGroup = this.group({});
            this.setupGroup(formGroup, options);
        }
        return formGroup;
    }
    /**
     * @param {?} form
     * @param {?} options
     * @return {?}
     */
    setupGroup(form, options) {
        for (const opt in options) {
            if (Array.isArray(options[opt])) {
                /** @type {?} */
                const controlOptions = (/** @type {?} */ (options[opt]));
                /** @type {?} */
                let validate = false;
                /** @type {?} */
                let asyncValidate = false;
                if (Array.isArray(controlOptions[1])) {
                    validate = true;
                }
                if (Array.isArray(controlOptions[2])) {
                    asyncValidate = true;
                }
                /** @type {?} */
                const ctrl = (/** @type {?} */ (new FormControl(controlOptions[0], validate ? controlOptions[1] : [], asyncValidate ? controlOptions[2] : [])));
                if (this.isObject(controlOptions[3])) {
                    /** @type {?} */
                    const ext = (/** @type {?} */ (options[opt][3]));
                    ctrl.errormsg = ext.errormsg;
                    ctrl.type = ext.type;
                    ctrl.label = ext.label;
                }
                form.addControl(opt, ctrl);
            }
            else {
                if (this.isObject(options[opt])) {
                    /** @type {?} */
                    const subG = this.group({});
                    form.addControl(opt, subG);
                    this.setupGroup((/** @type {?} */ (form.get(opt))), (/** @type {?} */ (options[opt])));
                }
            }
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    isObject(value) {
        return (typeof value === 'object' && value !== null && value.length === undefined);
    }
    /**
     * @param {?} val
     * @return {?}
     */
    isValidValue(val) {
        return (typeof val === 'string' ||
            typeof val === 'number' ||
            typeof val === 'boolean');
    }
    /**
     * @param {?} controlsConfig
     * @return {?}
     */
    _reduceControls(controlsConfig) {
        /** @type {?} */
        const controls = {};
        Object.keys(controlsConfig).forEach((/**
         * @param {?} controlName
         * @return {?}
         */
        controlName => {
            controls[controlName] = this._createControl(controlsConfig[controlName]);
        }));
        return controls;
    }
    /**
     * @param {?} controlConfig
     * @return {?}
     */
    _createControl(controlConfig) {
        if (controlConfig instanceof FormControl ||
            controlConfig instanceof FormGroup ||
            controlConfig instanceof FormArray) {
            return controlConfig;
        }
        else if (Array.isArray(controlConfig)) {
            /** @type {?} */
            const value = controlConfig[0];
            /** @type {?} */
            const validator = controlConfig.length > 1 ? controlConfig[1] : null;
            /** @type {?} */
            const asyncValidator = controlConfig.length > 2 ? controlConfig[2] : null;
            return this.control(value, validator, asyncValidator);
        }
        else {
            return this.control(controlConfig);
        }
    }
}
AutomaticFormBuilder.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AutomaticFormBuilder.ctorParameters = () => [
    { type: FormBuilder }
];
if (false) {
    /** @type {?} */
    AutomaticFormBuilder.prototype.array;
    /** @type {?} */
    AutomaticFormBuilder.prototype.control;
    /** @type {?} */
    AutomaticFormBuilder.prototype.group;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AutomaticFormbuilderModule {
}
AutomaticFormbuilderModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [ReactiveFormsModule],
                providers: [AutomaticFormBuilder],
                exports: [ReactiveFormsModule]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function IFormSettings() { }

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function IFormControlErrorMsg() { }

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function IExtFormControlSettings() { }
if (false) {
    /** @type {?} */
    IExtFormControlSettings.prototype.type;
    /** @type {?|undefined} */
    IExtFormControlSettings.prototype.label;
    /** @type {?|undefined} */
    IExtFormControlSettings.prototype.errormsg;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function IExtFormControl() { }
if (false) {
    /** @type {?} */
    IExtFormControl.prototype.errormsg;
    /** @type {?} */
    IExtFormControl.prototype.type;
    /** @type {?} */
    IExtFormControl.prototype.label;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AutomaticFormBuilder, AutomaticFormbuilderModule };
//# sourceMappingURL=ngx-automatic-formbuilder.js.map
